from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Sensor_Motion = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'ADXL343', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADXL343'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x5mm_P0.8mm_LayoutBorder1x6y'], 'footprint':'Package_LGA:LGA-14_3x5mm_P0.8mm_LayoutBorder1x6y', 'keywords':'3-axis accelerometer i2c spi mems', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/ADXL343.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nADXL343\n\n3-axis accelerometer i2c spi mems', 'pins':[
            Pin(num='1',name='Vdd_I/O',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='11',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SDO/ADDR',func=pin_types.BIDIR,unit=1),
            Pin(num='13',name='SDA/SDI/SDIO',func=pin_types.BIDIR,unit=1),
            Pin(num='14',name='SCL/SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='Vs',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'ADXL363', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADXL363'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:Analog_LGA-16_3.25x3mm_P0.5mm_LayoutBorder3x5y'], 'footprint':'Sensor_Motion:Analog_LGA-16_3.25x3mm_P0.5mm_LayoutBorder3x5y', 'keywords':'3-axis accelerometer spi mems thermometer', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/ADXL363.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nADXL363\n\n3-axis accelerometer spi mems thermometer', 'pins':[
            Pin(num='1',name='V_{DD_I/O}',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='13',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='V_{S}',func=pin_types.PWRIN,unit=1),
            Pin(num='16',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='ADC_IN',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='MOSI',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='MISO',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'BMF055', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'BMF055'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-28_5.2x3.8mm_P0.5mm'], 'footprint':'Package_LGA:LGA-28_5.2x3.8mm_P0.5mm', 'keywords':'9-axis motion sensor IMU SAMD20 ARM Cortex-M0+', 'description':'', 'datasheet':'https://www.bosch-sensortec.com/media/boschsensortec/downloads/datasheets/bst-bmf055-ds000.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nBMF055\n\n9-axis motion sensor IMU SAMD20 ARM Cortex-M0+', 'pins':[
            Pin(num='1',name='PB03(ACC_GYRO_INT2)',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='PA28',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='~{RESET}',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='(GYRO_CSB)PA27',func=pin_types.BIDIR,unit=1),
            Pin(num='13',name='PB23(ACC_GYRO_INT1)',func=pin_types.BIDIR,unit=1),
            Pin(num='14',name='PA24',func=pin_types.INPUT,unit=1),
            Pin(num='15',name='PA23',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='PA22',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='PA21',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='PA20',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='PB17',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='PB16',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='(MISO)PA19',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='(ACC_MAG_CSB)PA18',func=pin_types.BIDIR,unit=1),
            Pin(num='23',name='(SCLK)PA17',func=pin_types.BIDIR,unit=1),
            Pin(num='24',name='(MOSI)PA16',func=pin_types.BIDIR,unit=1),
            Pin(num='25',name='GNDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='26',name='PA01',func=pin_types.BIDIR,unit=1),
            Pin(num='27',name='PA00',func=pin_types.BIDIR,unit=1),
            Pin(num='28',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='PB02',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='PB01',func=pin_types.BIDIR,unit=1),
            Pin(num='6',name='PB00',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='(SWDIO)PA31',func=pin_types.BIDIR,unit=1),
            Pin(num='8',name='(SWCLK)PA30',func=pin_types.BIDIR,unit=1),
            Pin(num='9',name='CAP',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'BMI160', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'BMI160'}), 'ref_prefix':'U', 'fplist':['Package_LGA:Bosch_LGA-14_3x2.5mm_P0.5mm'], 'footprint':'Package_LGA:Bosch_LGA-14_3x2.5mm_P0.5mm', 'keywords':'Bosh IMU small low power inertial measurement unit', 'description':'', 'datasheet':'https://www.bosch-sensortec.com/media/boschsensortec/downloads/datasheets/bst-bmi160-ds000.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nBMI160\n\nBosh IMU small low power inertial measurement unit', 'pins':[
            Pin(num='1',name='SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='OCSB',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='OSDO',func=pin_types.BIDIR,unit=1),
            Pin(num='12',name='CSB',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SCx',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDx',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='ASDx',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='ASCx',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='INT1',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GNDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'BNO055', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'BNO055'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-28_5.2x3.8mm_P0.5mm'], 'footprint':'Package_LGA:LGA-28_5.2x3.8mm_P0.5mm', 'keywords':'IMU Sensor Fusion I2C UART', 'description':'', 'datasheet':'https://www.bosch-sensortec.com/media/boschsensortec/downloads/datasheets/bst-bno055-ds000.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nBNO055\n\nIMU Sensor Fusion I2C UART', 'pins':[
            Pin(num='1',name='PIN1',func=pin_types.NOCONNECT,unit=1),
            Pin(num='10',name='BL_IND',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',name='~{RESET}',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='PIN12',func=pin_types.NOCONNECT,unit=1),
            Pin(num='13',name='PIN13',func=pin_types.NOCONNECT,unit=1),
            Pin(num='14',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='15',name='GNDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='16',name='GNDIO',func=pin_types.PASSIVE,unit=1),
            Pin(num='17',name='COM3',func=pin_types.INPUT,unit=1),
            Pin(num='18',name='COM2',func=pin_types.PASSIVE,unit=1),
            Pin(num='19',name='COM1',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='COM0',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='PIN21',func=pin_types.NOCONNECT,unit=1),
            Pin(num='22',name='PIN22',func=pin_types.NOCONNECT,unit=1),
            Pin(num='23',name='PIN23',func=pin_types.NOCONNECT,unit=1),
            Pin(num='24',name='PIN24',func=pin_types.NOCONNECT,unit=1),
            Pin(num='25',name='GNDIO',func=pin_types.PASSIVE,unit=1),
            Pin(num='26',name='XOUT32',func=pin_types.OUTPUT,unit=1),
            Pin(num='27',name='XIN32',func=pin_types.INPUT,unit=1),
            Pin(num='28',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='~{BOOT_LOAD_PIN}',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='PS1',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='PS0',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='PIN7',func=pin_types.NOCONNECT,unit=1),
            Pin(num='8',name='PIN8',func=pin_types.NOCONNECT,unit=1),
            Pin(num='9',name='CAP',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'ICM-20602', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ICM-20602'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y'], 'footprint':'Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y', 'keywords':'accelerometer gyro mems motion', 'description':'', 'datasheet':'http://www.invensense.com/wp-content/uploads/2016/10/DS-000176-ICM-20602-v1.0.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nICM-20602\n\naccelerometer gyro mems motion', 'pins':[
            Pin(num='1',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='15',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='SAO/SD0',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='8',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'ICM-20948', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ICM-20948'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:InvenSense_QFN-24_3x3mm_P0.4mm'], 'footprint':'Sensor_Motion:InvenSense_QFN-24_3x3mm_P0.4mm', 'keywords':'mems magnetometer', 'description':'', 'datasheet':'http://www.invensense.com/wp-content/uploads/2016/06/DS-000189-ICM-20948-v1.3.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nICM-20948\n\nmems magnetometer', 'pins':[
            Pin(num='1',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='10',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='14',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='15',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='16',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='17',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='18',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='19',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='20',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='21',name='AUX_DA',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='23',name='SCL/SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='5',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='6',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='7',name='AUX_CL',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='SDO/AD0',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'IIM-42652', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'IIM-42652'}), 'ref_prefix':'U', 'fplist':['Package_LGA:Bosch_LGA-14_3x2.5mm_P0.5mm'], 'footprint':'Package_LGA:Bosch_LGA-14_3x2.5mm_P0.5mm', 'keywords':'IMU 6-Axis', 'description':'', 'datasheet':'https://invensense.tdk.com/wp-content/uploads/2021/01/ds-000440_iim-42652-datasheet.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nIIM-42652\n\nIMU 6-Axis', 'pins':[
            Pin(num='1',name='AP_SDO/AP_AD0',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='11',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='12',name='AP_CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='AP_SCL/AP_SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='AP_SDA/AP_SDIO/AP_SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='INT1/INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2/FSYNC/CLKIN',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'IIS3DWB', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'IIS3DWB'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'keywords':'low-noise vibration sensor', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/iis3dwb.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nIIS3DWB\n\nlow-noise vibration sensor', 'pins':[
            Pin(num='1',name='SDO',func=pin_types.OUTPUT,unit=1),
            Pin(num='10',name='RES',func=pin_types.NOCONNECT,unit=1),
            Pin(num='11',name='RES',func=pin_types.NOCONNECT,unit=1),
            Pin(num='12',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SPC',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDI',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDD_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'IPS2200', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'IPS2200'}), 'ref_prefix':'U', 'fplist':['Package_SO:TSSOP-16_4.4x5mm_P0.65mm'], 'footprint':'Package_SO:TSSOP-16_4.4x5mm_P0.65mm', 'keywords':'inductive position sensor', 'description':'', 'datasheet':'https://www.renesas.com/us/en/document/dst/ips2200-datasheet', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nIPS2200\n\ninductive position sensor', 'pins':[
            Pin(num='1',name='CSN_IRQN',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='11',name='OUT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='OUT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='OUT3',func=pin_types.OUTPUT,unit=1),
            Pin(num='14',name='OUT4',func=pin_types.OUTPUT,unit=1),
            Pin(num='15',name='SCK_SCL',func=pin_types.INPUT,unit=1),
            Pin(num='16',name='SIO_SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='RX1',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='RX2',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='RX3',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='RX4',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='TX1',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='TX2',func=pin_types.BIDIR,unit=1),
            Pin(num='8',name='VDDA',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'ISM330DHCX', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ISM330DHCX'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'keywords':'ST iNEMO IMU', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/ism330dhcx.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nISM330DHCX\n\nST iNEMO IMU', 'pins':[
            Pin(num='1',name='SDO/SA0',func=pin_types.OUTPUT,unit=1),
            Pin(num='10',name='OCS_AUX',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='SDO_AUX',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='SDx',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='SCx',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'KX022-1020', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'KX022-1020'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'3-axis accelerometer spi i2c mems', 'description':'', 'datasheet':'https://kionixfs.azureedge.net/en/datasheet/KX022-1020%20Specifications%20Rev%2012.0.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nKX022-1020\n\n3-axis accelerometer spi i2c mems', 'pins':[
            Pin(num='1',name='SDO/ADDR',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='NC',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SCLK/SCL',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='SDI/SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='IO_VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='TRIG',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'KXTJ3-1057', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'KXTJ3-1057'}), 'ref_prefix':'U', 'fplist':['Package_LGA:Kionix_LGA-12_2x2mm_P0.5mm_LayoutBorder2x4y'], 'footprint':'Package_LGA:Kionix_LGA-12_2x2mm_P0.5mm_LayoutBorder2x4y', 'keywords':'accelerometer tri-axis i2c', 'description':'', 'datasheet':'https://kionixfs.azureedge.net/en/datasheet/KXTJ3-1057-Specifications-Rev-5.0.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nKXTJ3-1057\n\naccelerometer tri-axis i2c', 'pins':[
            Pin(num='1',name='ADDR',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='VDD',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='IO_VDD',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='IO_VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='GND',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'L3GD20', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'L3GD20'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-16_4x4mm_P0.65mm_LayoutBorder4x4y'], 'footprint':'Package_LGA:LGA-16_4x4mm_P0.65mm_LayoutBorder4x4y', 'keywords':'3-Axis MEMS Gyroscope', 'description':'', 'datasheet':'http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00036465.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nL3GD20\n\n3-Axis MEMS Gyroscope', 'pins':[
            Pin(num='1',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='RES(GND)',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='RES(GND)',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='RES(GND)',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='14',name='RES(CAP)',func=pin_types.PASSIVE,unit=1),
            Pin(num='15',name='RES(VDD)',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='SDA/SDI/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='SDO/SA0',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='DRDY/INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='RES(GND)',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='RES(GND)',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LIS2DH', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LIS2DH'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_2x2mm_P0.35mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_2x2mm_P0.35mm_LayoutBorder3x4y', 'keywords':'3-axis accelerometer spi mems LGA-14', 'description':'', 'datasheet':'http://www.st.com/web/en/resource/technical/document/datasheet/DM00042751.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLIS2DH\n\n3-axis accelerometer spi mems LGA-14', 'pins':[
            Pin(num='1',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='SDO',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='Vdd_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='Vdd',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LIS2HH12', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LIS2HH12'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'3-axis accelerometer spi mems', 'description':'', 'datasheet':'www.st.com/resource/en/datasheet/lis2hh12.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLIS2HH12\n\n3-axis accelerometer spi mems', 'pins':[
            Pin(num='1',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='Vdd_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='11',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='SA0/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='Vdd',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LIS331HH', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LIS331HH'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y'], 'footprint':'Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y', 'keywords':'3-axis accelerometer spi i2c mems', 'description':'', 'datasheet':'https://media.digikey.com/pdf/Data%20Sheets/ST%20Microelectronics%20PDFS/LIS331HH.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLIS331HH\n\n3-axis accelerometer spi i2c mems', 'pins':[
            Pin(num='1',name='Vdd_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='Vdd',func=pin_types.PWRIN,unit=1),
            Pin(num='15',name='Vdd',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='SA0/SDO',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LIS3DH', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LIS3DH'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y'], 'footprint':'Package_LGA:LGA-16_3x3mm_P0.5mm_LayoutBorder3x5y', 'keywords':'3-axis accelerometer i2c spi mems', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/cd00274221.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLIS3DH\n\n3-axis accelerometer i2c spi mems', 'pins':[
            Pin(num='1',name='VDD_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='ADC3',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='15',name='ADC2',func=pin_types.INPUT,unit=1),
            Pin(num='16',name='ADC1',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='SPC',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='SDI',func=pin_types.INPUT,unit=1),
            Pin(num='7',name='SDO',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM303C', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM303C'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'Accelerometer Magnetometer MEMS', 'description':'', 'datasheet':'www.st.com/resource/en/datasheet/lsm303c.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM303C\n\nAccelerometer Magnetometer MEMS', 'pins':[
            Pin(num='1',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='11',name='DRDY_MAG',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='INT_XL',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='CS_XL',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='CS_MAG',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='SDA/SDI/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='C1',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='INT_MAG',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='VDD',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM303D', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM303D'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-16_3x3mm_P0.5mm'], 'footprint':'Package_LGA:LGA-16_3x3mm_P0.5mm', 'keywords':'Accelerometer Magnetometer MEMS', 'description':'', 'datasheet':'http://www.st.com/web/en/resource/technical/document/datasheet/DM00057547.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM303D\n\nAccelerometer Magnetometer MEMS', 'pins':[
            Pin(num='1',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='15',name='C1',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='2',name='SETC',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='SETP',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='SDA/SDI/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='SDO/SAO',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM303DLHC', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM303DLHC'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x5mm_P0.8mm_LayoutBorder1x6y'], 'footprint':'Package_LGA:LGA-14_3x5mm_P0.8mm_LayoutBorder1x6y', 'keywords':'Accelerometer Magnetometer MEMS', 'description':'', 'datasheet':'http://www.st.com/web/en/resource/technical/document/datasheet/DM00027543.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM303DLHC\n\nAccelerometer Magnetometer MEMS', 'pins':[
            Pin(num='1',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SETP',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='SETC',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='C1',func=pin_types.PASSIVE,unit=1),
            Pin(num='7',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='8',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='9',name='DRDY',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM6DS3', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM6DS3'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'keywords':'Accelerometer Gyroscope MEMS', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/lsm6ds3tr-c.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM6DS3\n\nAccelerometer Gyroscope MEMS', 'pins':[
            Pin(num='1',name='SDO/SA0',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='11',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='12',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='SDX',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='SCX',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM6DSM', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM6DSM'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'keywords':'Accelerometer Gyroscope MEMS', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/lsm6dsm.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM6DSM\n\nAccelerometer Gyroscope MEMS', 'pins':[
            Pin(num='1',name='SDO/SA0',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='OCS_Aux',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='SDO_Aux',func=pin_types.BIDIR,unit=1),
            Pin(num='12',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='SDX',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='SCX',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM9DS1', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM9DS1'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-24L_3x3.5mm_P0.43mm'], 'footprint':'Package_LGA:LGA-24L_3x3.5mm_P0.43mm', 'keywords':'I2C SPI IMU accelerometer gyroscope magnetometer', 'description':'', 'datasheet':'https://www.digikey.com/htmldatasheets/production/1639232/0/0/1/LSM9DS1-Datasheet.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM9DS1\n\nI2C SPI IMU accelerometer gyroscope magnetometer', 'pins':[
            Pin(num='1',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='INT_M',func=pin_types.OUTPUT,unit=1),
            Pin(num='11',name='INT1_A/G',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='INT2_A/G',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='DEN_A/G',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='RES',func=pin_types.BIDIR,unit=1),
            Pin(num='15',name='RES',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='RES',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='RES',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='RES',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='20',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='21',name='CAP',func=pin_types.PASSIVE,unit=1),
            Pin(num='22',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='23',name='VDD',func=pin_types.PASSIVE,unit=1),
            Pin(num='24',name='C1',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='VDDIO',func=pin_types.PASSIVE,unit=1),
            Pin(num='4',name='SDA/SDI/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='SDO_A/G',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='SDO_M',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='CS_A/G',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='CS_M',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='DRDY_M',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'MMA8653FCR1', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MMA8653FCR1'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:DFN-10_2x2mm_P0.4mm'], 'footprint':'Package_DFN_QFN:DFN-10_2x2mm_P0.4mm', 'keywords':'Accelerometer I2C', 'description':'', 'datasheet':'https://www.nxp.com/docs/en/data-sheet/MMA8653FC.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nMMA8653FCR1\n\nAccelerometer I2C', 'pins':[
            Pin(num='1',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='BYP',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='GND',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'MPU-6000', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MPU-6000'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm'], 'footprint':'Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm', 'keywords':'mems', 'description':'', 'datasheet':'https://invensense.tdk.com/wp-content/uploads/2015/02/MPU-6000-Datasheet1.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nMPU-6000\n\nmems', 'pins':[
            Pin(num='1',name='CLKIN',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='18',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='CPOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='23',name='SCL/SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='SDA/MOSI',func=pin_types.BIDIR,unit=1),
            Pin(num='6',name='AUX_DA',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='AUX_CL',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='9',name='AD0/MISO',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'MPU-6050', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MPU-6050'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm'], 'footprint':'Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm', 'keywords':'mems', 'description':'', 'datasheet':'https://invensense.tdk.com/wp-content/uploads/2015/02/MPU-6000-Datasheet1.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nMPU-6050\n\nmems', 'pins':[
            Pin(num='1',name='CLKIN',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='14',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='15',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='16',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='17',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='18',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='19',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='20',name='CPOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='21',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='22',name='RESV',func=pin_types.NOCONNECT,unit=1),
            Pin(num='23',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='5',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='6',name='AUX_DA',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='AUX_CL',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='VLOGIC',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='AD0',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'MPU-9150', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MPU-9150'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm'], 'footprint':'Sensor_Motion:InvenSense_QFN-24_4x4mm_P0.5mm', 'keywords':'mems magnetometer accelerometer gyroscope', 'description':'', 'datasheet':'https://www.invensense.com/wp-content/uploads/2015/02/MPU-9150-Datasheet.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nMPU-9150\n\nmems magnetometer accelerometer gyroscope', 'pins':[
            Pin(num='1',name='CLKIN',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='15',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='17',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='18',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='20',name='CPOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='23',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='ES_DA',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='ES_CL',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='VLOGIC',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='AD0',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'MPU-9250', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'MPU-9250'}), 'ref_prefix':'U', 'fplist':['Sensor_Motion:InvenSense_QFN-24_3x3mm_P0.4mm'], 'footprint':'Sensor_Motion:InvenSense_QFN-24_3x3mm_P0.4mm', 'keywords':'mems magnetometer', 'description':'', 'datasheet':'https://invensense.tdk.com/wp-content/uploads/2015/02/PS-MPU-9250A-01-v1.1.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nMPU-9250\n\nmems magnetometer', 'pins':[
            Pin(num='1',name='RESV_VDDIO',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='REGOUT',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='FSYNC',func=pin_types.INPUT,unit=1),
            Pin(num='12',name='INT',func=pin_types.OUTPUT,unit=1),
            Pin(num='13',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='18',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='RESV_GND',func=pin_types.PWRIN,unit=1),
            Pin(num='21',name='AUX_DA',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='23',name='SCL/SCLK',func=pin_types.INPUT,unit=1),
            Pin(num='24',name='SDA/MOSI',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='AUX_CL',func=pin_types.OUTPUT,unit=1),
            Pin(num='8',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='AD0/MISO',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'KX122-1042', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'KX122-1042'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm', 'Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'3-axis accelerometer spi i2c mems', 'description':'', 'datasheet':'http://kionixfs.kionix.com/en/datasheet/KX112-1042-Specifications-Rev-6.0.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nKX122-1042\n\n3-axis accelerometer spi i2c mems', 'pins':[
            Pin(num='1',name='SDO/ADDR',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='NC',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SCLK/SCL',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='SDI/SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='IO_VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='TRIG',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'KX222-1054', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'KX222-1054'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm', 'Package_LGA:LGA-12_2x2mm_P0.5mm', 'Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'3-axis accelerometer spi i2c mems', 'description':'', 'datasheet':'http://kionixfs.kionix.com/en/datasheet/KX222-1054-Specifications-Rev-2.0.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nKX222-1054\n\n3-axis accelerometer spi i2c mems', 'pins':[
            Pin(num='1',name='SDO/ADDR',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='NC',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='SCLK/SCL',func=pin_types.INPUT,unit=1),
            Pin(num='2',name='SDI/SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='IO_VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='TRIG',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='6',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='7',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='GND',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LIS2DE12', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LIS2DE12'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-12_2x2mm_P0.5mm', 'Package_LGA:LGA-12_2x2mm_P0.5mm'], 'footprint':'Package_LGA:LGA-12_2x2mm_P0.5mm', 'keywords':'3-axis accelerometer spi mems', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/lis2DE12.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLIS2DE12\n\n3-axis accelerometer spi mems', 'pins':[
            Pin(num='1',name='SCL/SPC',func=pin_types.INPUT,unit=1),
            Pin(num='10',name='Vdd_IO',func=pin_types.PWRIN,unit=1),
            Pin(num='11',name='INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='12',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='~{CS}',func=pin_types.INPUT,unit=1),
            Pin(num='3',name='SA0/SDO',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='SDA/SDI',func=pin_types.BIDIR,unit=1),
            Pin(num='5',name='RES',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='Vdd',func=pin_types.PWRIN,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'LSM6DSL', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'LSM6DSL'}), 'ref_prefix':'U', 'fplist':['Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y'], 'footprint':'Package_LGA:LGA-14_3x2.5mm_P0.5mm_LayoutBorder3x4y', 'keywords':'Accelerometer Gyroscope MEMS', 'description':'', 'datasheet':'https://www.st.com/resource/en/datasheet/lsm6dsl.pdf', 'search_text':'/usr/share/kicad/symbols/Sensor_Motion.kicad_sym\nLSM6DSL\n\nAccelerometer Gyroscope MEMS', 'pins':[
            Pin(num='1',name='SDO/SA0',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='11',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='12',name='CS',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='SDX',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='SCX',func=pin_types.INPUT,unit=1),
            Pin(num='4',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='VDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='6',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='VDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='INT2',func=pin_types.OUTPUT,unit=1)], 'unit_defs':[] })])


from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Fiber_Optic = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'AFBR-1624Z', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AFBR-1624Z'}), 'ref_prefix':'J', 'fplist':[''], 'footprint':'', 'keywords':'Fiber optic transmitter', 'description':'', 'datasheet':'https://docs.broadcom.com/docs/AV02-4369EN', 'search_text':'/usr/share/kicad/symbols/Fiber_Optic.kicad_sym\nAFBR-1624Z\n\nFiber optic transmitter', 'pins':[
            Pin(num='1',name='VCCT',func=pin_types.PWRIN,unit=1),
            Pin(num='2',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='3',name='VEET',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='Data_in',func=pin_types.INPUT,unit=1),
            Pin(num='5',name='HP',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='HP',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'AFBR-2624Z', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'AFBR-2624Z'}), 'ref_prefix':'J', 'fplist':[''], 'footprint':'', 'keywords':'fiber optic receiver', 'description':'', 'datasheet':'https://docs.broadcom.com/docs/AV02-4369EN', 'search_text':'/usr/share/kicad/symbols/Fiber_Optic.kicad_sym\nAFBR-2624Z\n\nfiber optic receiver', 'pins':[
            Pin(num='1',name='Data_Out',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='VEER',func=pin_types.PWRIN,unit=1),
            Pin(num='3',name='VCCR',func=pin_types.PWRIN,unit=1),
            Pin(num='4',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='5',name='HP',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='HP',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] })])


from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

MCU_Texas_SimpleLink = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'CC1312R1F3RGZ', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'CC1312R1F3RGZ'}), 'ref_prefix':'U', 'fplist':['Package_DFN_QFN:VQFN-48-1EP_7x7mm_P0.5mm_EP5.15x5.15mm'], 'footprint':'Package_DFN_QFN:VQFN-48-1EP_7x7mm_P0.5mm_EP5.15x5.15mm', 'keywords':'32-bit ARM Cortex-M4 MCU Microcontroller', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/cc1312r.pdf', 'search_text':'/usr/share/kicad/symbols/MCU_Texas_SimpleLink.kicad_sym\nCC1312R1F3RGZ\n\n32-bit ARM Cortex-M4 MCU Microcontroller', 'pins':[
            Pin(num='1',name='RF_P',func=pin_types.PASSIVE,unit=1),
            Pin(num='10',name='DIO_5',func=pin_types.BIDIR,unit=1),
            Pin(num='11',name='DIO_6',func=pin_types.BIDIR,unit=1),
            Pin(num='12',name='DIO_7',func=pin_types.BIDIR,unit=1),
            Pin(num='13',name='VDDS2',func=pin_types.PWRIN,unit=1),
            Pin(num='14',name='DIO_8',func=pin_types.BIDIR,unit=1),
            Pin(num='15',name='DIO_9',func=pin_types.BIDIR,unit=1),
            Pin(num='16',name='DIO_10',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='DIO_11',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='DIO_12',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='DIO_13',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='RF_N',func=pin_types.PASSIVE,unit=1),
            Pin(num='20',name='DIO_14',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='DIO_15',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='VDDS3',func=pin_types.PWRIN,unit=1),
            Pin(num='23',name='DCOUPL',func=pin_types.PWROUT,unit=1),
            Pin(num='24',name='JTAG_TMSC',func=pin_types.BIDIR,unit=1),
            Pin(num='25',name='JTAG_TCKC',func=pin_types.INPUT,unit=1),
            Pin(num='26',name='DIO_16/JTAG_TDO',func=pin_types.BIDIR,unit=1),
            Pin(num='27',name='DIO_17/JTAG_TDI',func=pin_types.BIDIR,unit=1),
            Pin(num='28',name='DIO_18',func=pin_types.BIDIR,unit=1),
            Pin(num='29',name='DIO_19',func=pin_types.BIDIR,unit=1),
            Pin(num='3',name='RX_TX',func=pin_types.PASSIVE,unit=1),
            Pin(num='30',name='DIO_20',func=pin_types.BIDIR,unit=1),
            Pin(num='31',name='DIO_21',func=pin_types.BIDIR,unit=1),
            Pin(num='32',name='DIO_22',func=pin_types.BIDIR,unit=1),
            Pin(num='33',name='DCDC_SW',func=pin_types.PWROUT,unit=1),
            Pin(num='34',name='VDDS_DCDC',func=pin_types.PWRIN,unit=1),
            Pin(num='35',name='~{RESET}',func=pin_types.INPUT,unit=1),
            Pin(num='36',name='DIO_23',func=pin_types.BIDIR,unit=1),
            Pin(num='37',name='DIO_24',func=pin_types.BIDIR,unit=1),
            Pin(num='38',name='DIO_25',func=pin_types.BIDIR,unit=1),
            Pin(num='39',name='DIO_26',func=pin_types.BIDIR,unit=1),
            Pin(num='4',name='X32K_Q1',func=pin_types.INPUT,unit=1),
            Pin(num='40',name='DIO_27',func=pin_types.BIDIR,unit=1),
            Pin(num='41',name='DIO_28',func=pin_types.BIDIR,unit=1),
            Pin(num='42',name='DIO_29',func=pin_types.BIDIR,unit=1),
            Pin(num='43',name='DIO_30',func=pin_types.BIDIR,unit=1),
            Pin(num='44',name='VDDS',func=pin_types.PWRIN,unit=1),
            Pin(num='45',name='VDDR',func=pin_types.PWRIN,unit=1),
            Pin(num='46',name='X48M_N',func=pin_types.INPUT,unit=1),
            Pin(num='47',name='X48M_P',func=pin_types.INPUT,unit=1),
            Pin(num='48',name='VDDR_RF',func=pin_types.PWRIN,unit=1),
            Pin(num='49',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='5',name='X32K_Q2',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='DIO_1',func=pin_types.BIDIR,unit=1),
            Pin(num='7',name='DIO_2',func=pin_types.BIDIR,unit=1),
            Pin(num='8',name='DIO_3',func=pin_types.BIDIR,unit=1),
            Pin(num='9',name='DIO_4',func=pin_types.BIDIR,unit=1)], 'unit_defs':[] })])


"""
Mixin classes for SKiDL objects.

This module provides mixin classes that add specific functionality to SKiDL objects.
The mixins are designed to be combined with base classes to provide additional
capabilities without requiring deep inheritance hierarchies.

Classes:
    PinMixin: Adds pin management functionality to parts and other objects.
"""

import sys
from .logger import active_logger
from .skidlbaseobj import SkidlBaseObject
from .utilities import (
    expand_indices,
    filter_list,
    flatten,
    from_iadd,
    list_or_scalar,
    rmv_iadd,
    Rgx,
)

class PinMixin():
    """
    Mixin class that adds pin-related methods and functionality to a class.
    
    This mixin provides comprehensive pin management capabilities including:
    - Adding, removing, and manipulating pins
    - Pin selection using various criteria (names, numbers, regex patterns)
    - Pin connection and disconnection operations
    - Pin aliasing and naming utilities
    
    The mixin maintains a list of pins and provides multiple ways to access them,
    including bracket notation, attribute access, and iteration.
    
    Attributes:
        pins (list): List of Pin objects belonging to the part.
        _match_pin_regex (bool): Enable/disable regex matching for pin names and aliases.
    
    Examples:
        >>> class MyPart(PinMixin):
        ...     def __init__(self):
        ...         super().__init__()
        ...         # Add pins to the part
        ...         self.add_pins(Pin(num=1, name='VCC'), Pin(num=2, name='GND'))
        ...
        >>> part = MyPart()
        >>> vcc_pin = part[1]  # Get pin by number
        >>> gnd_pin = part['GND']  # Get pin by name
    """

    def __init__(self):
        """
        Initialize the PinMixin.
        
        Sets up the pin list and configures default settings for pin matching.
        """
        self.pins = []  # List of pins in the part.
        self._match_pin_regex = False  # Disable regex matching of pin names/aliases by default.

    def __iadd__(self, *pins):
        """
        Add one or more pins to the part using the += operator.
        
        This method allows pins to be added using the += operator syntax,
        which is more intuitive than calling add_pins() directly.
        
        Args:
            *pins: Variable number of Pin objects to add to the part.
            
        Returns:
            Self: The part object with added pins, allowing for method chaining.
            
        Examples:
            >>> part += Pin(num=1, name='VCC')
            >>> part += [Pin(num=2, name='GND'), Pin(num=3, name='DATA')]
        """
        return self.add_pins(*pins)
    

    # Get pins from a part using brackets, e.g. [1,5:9,'A[0-9]+'].
    def __getitem__(self, *pin_ids, **criteria):
        """
        Return pins selected by pin numbers, names, or other criteria.

        This method enables bracket notation for pin selection, supporting
        various selection methods including exact matches, regex patterns,
        and attribute-based filtering.

        Args:
            *pin_ids: Pin identifiers which can be:
                - integers or strings for pin numbers
                - strings for pin names
                - regex patterns (if regex matching is enabled)
                - slices for ranges of pins
                - lists or tuples of any of the above
                If empty, selects all pins.

        Keyword Args:
            criteria: Key/value pairs specifying pin attributes that must match.
                Common criteria include 'func' for pin function, 'unit' for
                unit number, etc.

        Returns:
            Pin, NetPinList, or None: 
                - Single Pin if exactly one match found
                - NetPinList if multiple matches found  
                - None if no matches found

        Examples:
            >>> pin1 = part[1]  # Get pin by number
            >>> reset_pin = part['RESET']  # Get pin by name
            >>> power_pins = part['VCC', 'VDD', 'GND']  # Multiple pins
            >>> analog_pins = part[func='analog']  # Pins with specific function
        """
        return self.get_pins(*pin_ids, **criteria)

    def __setitem__(self, ids, pins_nets_buses):
        """
        Prevent direct assignment to pins while allowing += operator.

        This method is a work-around that allows the use of the += for making
        connections to pins while prohibiting direct assignment. Python
        processes something like my_part['GND'] += gnd as follows::

            1. Part.__getitem__ is called with 'GND' as the index. This
               returns a single Pin or a NetPinList.
            2. The Pin.__iadd__ or NetPinList.__iadd__ method is passed
               the thing to connect to the pin (gnd in this case). This method
               makes the actual connection to the part pin or pins. Then it
               creates an iadd_flag attribute in the object it returns.
            3. Finally, Part.__setitem__ is called. If the iadd_flag attribute
               is true in the passed argument, then __setitem__ was entered
               as part of processing the += operator. If there is no
               iadd_flag attribute, then __setitem__ was entered as a result
               of using a direct assignment, which is not allowed.
               
        Args:
            ids: Pin identifiers being assigned to.
            pins_nets_buses: Object being assigned to the pins.
            
        Raises:
            TypeError: If direct assignment is attempted (no iadd_flag present).
            
        Note:
            This is part of Python's mechanism for handling augmented assignment
            operators. The += operator first calls __getitem__, then __iadd__
            on the returned object, then __setitem__ with the result.
        """
        # If the iadd_flag is set, then it's OK that we got
        # here and don't issue an error. Also, delete the flag.
        if from_iadd(pins_nets_buses):
            rmv_iadd(pins_nets_buses)
            return

        # No iadd_flag or it wasn't set. This means a direct assignment
        # was made to the pin, which is not allowed.
        active_logger.raise_(TypeError, "Can't assign to a part! Use the += operator.")

    def __getattr__(self, attr):
        """
        Enable attribute-style access to pins using their aliases.
        
        When a normal attribute lookup fails, this method searches for pins
        that have the requested attribute name as an alias. This allows
        pins to be accessed as attributes of the part.
        
        Args:
            attr (str): The attribute name being requested.
            
        Returns:
            Pin or NetPinList: 
                - Single Pin if exactly one pin has this alias
                - NetPinList if multiple pins have this alias
            
        Raises:
            AttributeError: If no pins have the requested alias and the
                attribute doesn't exist in the base class.
                
        Examples:
            >>> part.RESET  # Access pin with 'RESET' alias
            >>> part.p1     # Access pin 1 using 'p1' alias
        """
        from skidl.netpinlist import NetPinList

        # Look for the attribute name in the list of pin aliases.
        pins = [pin for pin in self if pin.aliases == attr]

        if pins:
            if len(pins) == 1:
                # Return a single pin if only one alias match was found.
                return pins[0]
            else:
                # Return list of pins if multiple matches were found.
                # Return a NetPinList instead of a vanilla list so += operator works!
                return NetPinList(pins)

        # No pin aliases matched, so use the __getattr__ for the subclass.
        # Don't use super(). It leads to long runtimes under Python 2.7.
        return SkidlBaseObject.__getattr__(self, attr)

    def __iter__(self):
        """
        Enable iteration over the part's pins.
        
        This method makes the part object iterable, allowing direct iteration
        over its pins using for loops and other iteration constructs.
        
        Returns:
            generator: Generator expression yielding Pin objects.
            
        Examples:
            >>> for pin in part:
            ...     print(f"Pin {pin.num}: {pin.name}")
            >>> pin_names = [pin.name for pin in part]
        """

        # Get the list pf pins for this part using the getattribute for the
        # basest object to prevent infinite recursion within the __getattr__ method.
        # Don't use super() because it leads to long runtimes under Python 2.7.
        self_pins = object.__getattribute__(self, "pins")

        return (p for p in self_pins)  # Return generator expr.

    def associate_pins(self):
        """
        Ensure all pins have proper back-references to this part.
        
        This method updates each pin's 'part' attribute to point back to
        this part object, maintaining bidirectional relationships between
        parts and their pins. This is typically called after pins are
        added or when the part structure is modified.
        """
        for p in self:
            p.part = self

    def add_pins(self, *pins):
        """
        Add one or more pins to the part.
        
        This method adds pins to the part and sets up proper relationships
        and aliases. Each pin gets a back-reference to the part, and 
        automatic aliases are created for pin names and numbers.
        
        Args:
            *pins: Variable number of Pin objects or iterables of Pin objects
                to add to the part.
            
        Returns:
            Self: The part object with pins added, enabling method chaining.
            
        Note:
            Automatic aliases are created:
            - Pin name becomes an alias
            - "p" + pin number becomes an alias (e.g., "p1" for pin 1)
            
        Examples:
            >>> part.add_pins(Pin(num=1, name='VCC'))
            >>> part.add_pins([Pin(num=2, name='GND'), Pin(num=3, name='DATA')])
        """
        for pin in flatten(pins):
            pin.part = self
            self.pins.append(pin)
        return self

    def create_pins(self, base_name, pin_count=None, connections=None):
        """
        Create one or more pins with systematic naming and numbering.
        
        This method creates multiple pins with names based on a common prefix
        and systematic numbering. Pins can optionally be connected to provided
        nets, buses, or other pins during creation.
        
        Args:
            base_name (str): Base name for the pins. Pin indices will be appended
                to this name (e.g., "DATA" becomes "DATA1", "DATA2", etc.).
            pin_count (int, range, slice, list, tuple, or None): Specification of
                how many pins to create and their numbering:
                - int N: Creates N pins numbered 1 to N
                - range or slice: Uses the range values for pin numbers
                - tuple or list: Uses the provided list of pin numbers
                - None: Creates single pin without index appended to name
            connections (list or scalar, optional): List of nets, buses, or pins to connect
                to each created pin. Must match the number of pins created.
                A scalar Net, Pin, or Bus will will be converted to a list of one item.
        
        Returns:
            Self: The part object with pins created, enabling method chaining.
            
        Raises:
            ValueError: If connections list length doesn't match number of pins created.
            
        Examples:
            >>> part.create_pins("DATA", 8)  # Creates DATA1, DATA2, ..., DATA8
            >>> part.create_pins("ADDR", range(0, 16))  # Creates ADDR0 to ADDR15
            >>> part.create_pins("CLK", None)  # Creates single pin named "CLK"
            >>> part.create_pins("IO", 4, [net1, net2, net3, net4])  # With connections
        """
        from .netpinlist import NetPinList
        from .part import Part
        from .pin import Pin, PhantomPin

        if connections is not None:
            # This converts a single Net or Pin to a list but leaves a list unaltered.
            # It also converts a Bus to a list of its nets.
            connections = NetPinList(connections)

        # Determine pin indices based on pin_count parameter
        if pin_count is None:
            if connections is None or len(connections)==1:
                # Single pin without index
                indices = [None]
            else:
                # Create as many pins as there are connections
                indices = range(1, len(connections) + 1)
        elif isinstance(pin_count, int):
            # Integer N creates pins 1 to N
            indices = range(1, pin_count + 1)
        elif isinstance(pin_count, (range, slice, list, tuple)):
            # Use range/slice directly
            if isinstance(pin_count, slice):
                # Convert slice to range
                start = pin_count.start if pin_count.start is not None else 1
                stop = pin_count.stop
                step = pin_count.step if pin_count.step is not None else 1
                indices = range(start, stop, step)
            else:
                indices = pin_count
        else:
            active_logger.error(f"pin_count must be int, range, slice, or None, got {type(pin_count)}")
            return self
        
        # Validate connections list length if provided
        if connections is not None:
            if len(connections) != len(indices):
                active_logger.error(
                    f"Number of connections ({len(connections)}) must match "
                    f"number of pins created ({len(indices)})"
                )
                return self
        
        # Determine the class of pins to create.
        if isinstance(self, Part):
            pin_class = Pin  # Create regular pins for Parts.
        else:
            pin_class = PhantomPin  # Use PhantomPins for I/O of SubCircuit or Interface

        # Create the pins
        created_pins = []
        for index in indices:
            # Generate pin name
            pin_num = len(self.pins) + 1  # Use next available number
            if index is None:
                pin_name = base_name
            else:
                pin_name = f"{base_name}{index}"
            
            # Create the pin.
            pin = pin_class(num=pin_num, name=pin_name, part=self)

            # Add the pin to the part so pin numbering is correct.
            self.add_pins(pin)

            created_pins.append(pin)
        
        # Connect created pins if connections provided
        if connections:
            connections += created_pins

        return self

    def rmv_pins(self, *pin_ids):
        """
        Remove one or more pins from the part.
        
        Removes pins that match the given identifiers (names or numbers).
        The pins are permanently removed from the part's pin list.
        
        Args:
            *pin_ids: Pin identifiers (names or numbers) of pins to remove.
            
        Examples:
            >>> part.rmv_pins(1, 'RESET')  # Remove pin 1 and RESET pin
            >>> part.rmv_pins('VCC', 'GND')  # Remove power pins
        """

        # Expand pin_ids to a flat list of individual pin numbers/names.
        pin_ids = list(set(expand_indices(0, sys.maxsize, False, *pin_ids)))

        # All pin numbers and names are stored as strings, so convert ids to strings.
        pin_ids = [str(pin_id) for pin_id in pin_ids]

        # Remove pins in reverse order to avoid index shifting issues.
        for i, pin in reversed(tuple(enumerate(self))):
            # Look for an intersection of the pin name and aliases with the pin_ids to remove.
            pin_id = set((pin.num, *pin.aliases))
            if not pin_id.isdisjoint(pin_ids):
                del self.pins[i]

    def swap_pins(self, pin_id1, pin_id2):
        """
        Swap the names and numbers between two pins.
        
        This method exchanges the name and number attributes between two pins,
        effectively swapping their identities while maintaining their physical
        connections and other properties.
        
        Args:
            pin_id1: Identifier (name or number) of the first pin.
            pin_id2: Identifier (name or number) of the second pin.
            
        Examples:
            >>> part.swap_pins(1, 2)  # Swap pins 1 and 2
            >>> part.swap_pins('RESET', 'ENABLE')  # Swap named pins
        """

        # All pin numbers and names are stored as strings, so convert ids to strings.
        pin_id1 = str(pin_id1)
        pin_id2 = str(pin_id2)

        pins = self.pins
        i1, i2 = None, None
        for i, pin in enumerate(pins):
            pin_num_name = (pin.num, *pin.aliases)
            if pin_id1 in pin_num_name:
                i1 = i
            elif pin_id2 in pin_num_name:
                i2 = i
            if i1!=None and i2!=None:
                # Found both pins, so swap pin numbers and names
                pins[i1].num, pins[i1].name, pins[i2].num, pins[i2].name = (
                    pins[i2].num,
                    pins[i2].name,
                    pins[i1].num,
                    pins[i1].name,
                )
                # The swap has been made, so we're done.
                return

    def rename_pin(self, pin_id, new_pin_name):
        """
        Change the name of a pin.
        
        Finds the pin matching the given identifier and updates its name
        to the new value.
        
        Args:
            pin_id: Current identifier (name or number) of the pin to rename.
            new_pin_name (str): New name to assign to the pin.
            
        Examples:
            >>> part.rename_pin(1, 'POWER')  # Rename pin 1 to 'POWER'
            >>> part.rename_pin('RESET', 'RST')  # Rename RESET pin to RST
        """

        # All pin numbers and names are stored as strings, so convert id to string.
        pin_id = str(pin_id)
        for pin in self:
            if pin_id in (pin.num, *pin.aliases):
                # Found pin so change its name
                pin.name = new_pin_name
                return

    def renumber_pin(self, pin_id, new_pin_num):
        """
        Change the number of a pin.
        
        Finds the pin matching the given identifier and updates its number
        to the new value.
        
        Args:
            pin_id: Current identifier (name or number) of the pin to renumber.
            new_pin_num: New number to assign to the pin.
            
        Examples:
            >>> part.renumber_pin('RESET', 100)  # Change RESET pin to number 100
            >>> part.renumber_pin(1, 5)  # Change pin 1 to pin 5
        """

        # All pin numbers and names are stored as strings, so convert id to string.
        pin_id = str(pin_id)
        for pin in self:
            if pin_id in (pin.num, *pin.aliases):
                # Found pin so change its number
                pin.num = new_pin_num
                return

    def get_pins(self, *pin_ids, **criteria):
        """
        Get pins matching specified identifiers and criteria.

        This is the core pin selection method that supports multiple selection
        modes including exact matching, regex patterns, and attribute-based
        filtering. It provides flexible pin selection capabilities for various
        use cases.

        Args:
            *pin_ids: Pin identifiers for selection:
                - Integers or strings for exact pin number matches
                - Strings for exact pin name/alias matches  
                - Regex patterns (when regex matching enabled)
                - Slices for pin number ranges
                - Lists/tuples of any combination above
                If empty, selects all pins.

        Keyword Args:
            criteria: Attribute-based filtering criteria as key=value pairs.
            silent (bool, optional): Suppress error messages if True. Defaults to False.
            only_search_numbers (bool, optional): Restrict search to pin numbers only. 
                Defaults to False.
            only_search_names (bool, optional): Restrict search to pin names/aliases only.
                Defaults to False.
            match_regex (bool, optional): Enable regex pattern matching for names.
                Defaults to False, or uses part's match_pin_regex setting.

        Returns:
            Pin, list, or None:
                - Single Pin object if exactly one match found
                - List of Pin objects if multiple matches found
                - None if no matches found and silent=True
                
        Raises:
            ValueError: If no pins found and silent=False.
            
        Examples:
            >>> pins = part.get_pins(1, 2, 3)  # Get pins 1, 2, 3
            >>> analog_pins = part.get_pins(func='analog')  # Pins with analog function
            >>> power_pins = part.get_pins('VCC', 'VDD', 'GND')  # Named pins
            >>> pattern_pins = part.get_pins('A[0-9]+', match_regex=True)  # Regex
        """

        from .alias import Alias
        from .netpinlist import NetPinList

        # Extract option for suppressing error messages.
        silent = criteria.pop("silent", False)

        # Extract restrictions on searching for only pin names or numbers.
        only_search_numbers = criteria.pop("only_search_numbers", False)
        only_search_names = criteria.pop("only_search_names", False)

        # Extract permission to search for regex matches in pin names/aliases.
        match_regex = criteria.pop("match_regex", False) or self.match_pin_regex

        # If no pin identifiers were given, then use a wildcard that will
        # select all pins.
        if not pin_ids:
            pin_ids = [Rgx(".*")]

        # Determine the minimum and maximum pin ids if they don't already exist.
        if "min_pin" not in dir(self) or "max_pin" not in dir(self):
            self.min_pin, self.max_pin = self._find_min_max_pins()

        # Go through the list of pin IDs one-by-one.
        pins = NetPinList()
        for p_id in expand_indices(self.min_pin, self.max_pin, match_regex, *pin_ids):

            # If only names are being searched, the search of pin numbers is skipped.
            if not only_search_names:
                # Does pin ID (either integer or string) match a pin number...
                tmp_pins = filter_list(
                    self.pins, num=str(p_id), do_str_match=True, **criteria
                )
                if tmp_pins:
                    pins.extend(tmp_pins)
                    continue

            # if only numbers are being searched, then search of pin names is skipped.
            if not only_search_numbers:
                # OK, assume it's not a pin number but a pin name or alias.
                # Look for an exact match.

                # Check pin aliases for an exact match.
                tmp_pins = filter_list(
                    self.pins, aliases=p_id, do_str_match=True, **criteria
                )
                if tmp_pins:
                    pins.extend(tmp_pins)
                    continue

                # Check pin names for an exact match.
                tmp_pins = filter_list(
                    self.pins, name=p_id, do_str_match=True, **criteria
                )
                if tmp_pins:
                    pins.extend(tmp_pins)
                    continue

                # Skip regex matching if not enabled.
                if not match_regex:
                    continue

                # OK, pin ID is not a pin number and doesn't exactly match a pin
                # name or alias. Does it match as a regex?
                p_id_re = p_id

                # Check pin aliases for a regex match.
                tmp_pins = filter_list(self.pins, aliases=Alias(p_id_re), **criteria)
                if tmp_pins:
                    pins.extend(tmp_pins)
                    continue

                # Check the pin names for a regex match.
                tmp_pins = filter_list(self.pins, name=p_id_re, **criteria)
                if tmp_pins:
                    pins.extend(tmp_pins)
                    continue

        # Log an error if no pins were selected using the pin ids.
        if not pins and not silent:
            active_logger.error(
                f"No pins found using {self.name}:{self.ref}[{pin_ids}]"
            )

        return list_or_scalar(pins)

    def disconnect(self):
        """
        Disconnect all pins from their connected nets.
        
        This method breaks all electrical connections to the part by
        disconnecting each pin from any nets it may be connected to.
        The part becomes electrically isolated after this operation.
        
        Examples:
            >>> part.disconnect()  # Disconnect all pins from nets
        """
        for pin in self:
            pin.disconnect()

    def split_pin_names(self, delimiters):
        """
        Split pin names using delimiters and add subnames as aliases.
        
        This method takes pin names that contain delimiter characters and
        splits them into component parts, adding each part as an alias
        to the pin. This enables more flexible pin access patterns.
        
        Args:
            delimiters (str): String containing characters to use as delimiters
                for splitting pin names.
                
        Examples:
            >>> part.split_pin_names('_-/')  # Split on underscore, dash, slash
            >>> # Pin named "DATA_IN" would get aliases "DATA" and "IN"
        """
        if delimiters:
            for pin in self:
                # Split pin name and add subnames as aliases to the pin.
                pin.split_name(delimiters)

    def _find_min_max_pins(self):
        """
        Find the minimum and maximum numeric pin numbers.
        
        This internal method scans all pins to find the lowest and highest
        numbered pins (considering only pins with integer numbers). These
        values are used for pin range operations and indexing.
        
        Returns:
            tuple: A tuple of (min_pin_number, max_pin_number) as integers.
                Returns (0, 0) if no numeric pins are found.
        """
        pin_nums = []
        try:
            for p in self:
                try:
                    pin_nums.append(int(p.num))
                except ValueError:
                    pass
        except AttributeError:
            # This happens if the part has no pins.
            pass
        try:
            return min(pin_nums), max(pin_nums)
        except ValueError:
            # This happens if the part has no integer-labeled pins.
            return 0, 0

    @property
    def ordered_pins(self):
        """
        Get the pins sorted in a consistent order.
        
        Returns the part's pins in sorted order, typically by pin number
        where possible, falling back to name-based sorting for non-numeric pins.
        
        Returns:
            list: Sorted list of the part's Pin objects.
            
        Examples:
            >>> sorted_pins = part.ordered_pins
            >>> for pin in part.ordered_pins:
            ...     print(f"Pin {pin.num}: {pin.name}")
        """
        return sorted(self)

    @property
    def match_pin_regex(self):
        """
        Get the enable/disable flag for pin regular-expression matching.
        
        Returns:
            bool: Current state of regex matching flag.
        """
        return self._match_pin_regex

    @match_pin_regex.setter
    def match_pin_regex(self, flag):
        """
        Set the regex matching flag.
        
        Args:
            flag (bool): True to enable regex matching for pins, False to disable.
            
        Notes:
            This also sets the flag for all units of the part.
        """
        self._match_pin_regex = flag

        # Also set flag for units of the part.
        for unit in self.unit.values():
            unit._match_pin_regex = flag

    @match_pin_regex.deleter
    def match_pin_regex(self):
        """
        Delete the regex matching flag.
        """
        del self._match_pin_regex



# -*- coding: utf-8 -*-

# The MIT License (MIT) - Copyright (c) Dave Vandenbout.

"""
Functions for generating a KiCad PCB.
"""

from skidl.scriptinfo import get_script_name
from skidl.utilities import export_to_all
from skidl.logger import active_logger


@export_to_all
def gen_pcb(circuit, pcb_file, fp_libs=None):
    """Create a KiCad PCB file directly from a Circuit object.

    Args:
        circuit (Circuit): Circuit object.
        pcb_file: Either a file object that can be written to, or a string
            containing a file name, or None.
        fp_libs: List of directories containing footprint libraries.
    Returns:
        None.
    """

    # Keep the import in here so it doesn't get triggered unless this is used
    # so it eases some problems with tox testing.
    # It requires pcbnew module which may not be present or may be for the
    # wrong Python version (2 vs. 3).
    try:
        import kinet2pcb  # For creating KiCad PCB directly from Circuit object.
    except ImportError:
        active_logger.warning(
            "kinet2pcb module is missing. Can't generate a KiCad PCB without it."
        )
    else:
        pcb_file = pcb_file or (get_script_name() + ".kicad_pcb")
        kinet2pcb.kinet2pcb(circuit, pcb_file, fp_libs)



# -*- coding: utf-8 -*-

# The MIT License (MIT) - Copyright (c) Dave Vandenbout.

"""
Interface management in SKiDL.

This module provides the Interface class, which bundles nets, buses, and pins into
a single entity that can be used for connecting subsystems with complex I/O.
Interfaces make it easier to handle groups of related signals and enable 
connecting subsystems with matching interfaces using simple operators.
"""

from .alias import Alias
from .bus import Bus
from .net import Net
from .netpinlist import NetPinList
from .pin import Pin
from .skidlbaseobj import SkidlBaseObject
from .utilities import (
    expand_indices,
    export_to_all,
    filter_list,
    from_iadd,
    list_or_scalar,
    rmv_iadd,
    set_iadd,
)



@export_to_all
class Interface(dict):
    """
    A bundle of nets, buses, and pins that can be accessed as attributes or dict entries.
    
    An Interface groups related nets, buses, and pins into a single entity where each
    item can be accessed either as an attribute (obj.item) or as a dictionary key (obj['item']).
    This makes Interfaces useful for standardizing connections between subsystems or
    for creating reusable connection patterns.
    
    Args:
        *args: Positional arguments passed to dict constructor.
        **kwargs: Keyword arguments defining nets/buses to include in the interface.
        
    Keyword Args:
        unbundle (bool, optional): Whether to automatically unbundle buses into individual
            nets that can be accessed directly. Defaults to True.
            
    Examples:
        >>> power = Interface(vcc=Net('VCC'), gnd=Net('GND'))
        >>> power.vcc += part1['VCC']  # Connect using attribute access
        >>> power['gnd'] += part1['GND']  # Connect using dictionary access
    """

    # Set the default ERC functions for all Interface instances.
    erc_list = []

    def __init__(self, *args, **kwargs):

        # By default, buses are unbundled into individual nets.
        unbundle = kwargs.pop("unbundle", True)

        self.unexpio = dict()

        # Start with a standard dictionary of objects.
        super().__init__(*args, **kwargs)

        super().__setattr__("match_pin_regex", False)

        for k, v in list(self.items()):
            if isinstance(v, (Pin, Net, Bus, NetPinList, SkidlBaseObject)):
                # Add SKiDL-type objects.
                self.__setattr__(k, v, unbundle=unbundle)
            else:
                # Add standard Python objects.
                super().__setattr__(k, v)

    def __setattr__(self, key, value, unbundle=True):
        """
        Set an attribute and corresponding dict entry in the interface.
        
        This method handles creating net-like objects for pin-like inputs,
        setting up aliases for the keys, and optionally unbundling buses
        into individual nets.
        
        Args:
            key (str): Name for the attribute/dict entry.
            value: The value to associate with the key.
            unbundle (bool, optional): Whether to unbundle buses. Defaults to True.
        """

        # Create net-like objects for pin-like objects.
        if isinstance(value, NetPinList):
            # Convert NetPinList into a Bus.
            value = Bus(value)
        elif isinstance(value, Pin):
            # Convert Pin into a Net.
            n = Net()
            n += value
            value = n

        # Assign the key as an alias to any net-like object.
        if isinstance(value, (Net, Bus)):
            value.aliases += key
            self.unexpio[key] = value

        # Add the value to the dictionary and as an attribute.
        if isinstance(value, SkidlBaseObject):
            # Only SKiDL-type objects get added as dictionary items.
            super().__setitem__(key, value)
        super().__setattr__(key, value)

        # If enabled, expand a bus and add its individual nets.
        if isinstance(value, Bus) and unbundle:
            for i, v in enumerate(value):
                n = Net(circuit=v.circuit)
                n.aliases += key + str(i)
                n += v
                super().__setitem__(key + str(i), n)
                super().__setattr__(key + str(i), n)
                # self.setattr(self, key + str(i), n)

    def __getitem__(self, *io_ids, **criteria):
        """
        Get interface items by name, regex, or other criteria.
        
        This method allows retrieving individual items or groups of items
        from the interface based on names, regular expressions, or other
        attribute criteria.
        
        Args:
            io_ids: Names or patterns of interface items to retrieve.
                If empty, all items may be retrieved.
                
        Keyword Args:
            match_regex (bool, optional): Whether to use regex matching. Default is False.
            criteria: Attribute values that items must match to be retrieved.
            
        Returns:
            Net, Pin, Bus, NetPinList, or None: The selected item(s) or None if no match.
        """

        # Extract permission to search for regex matches in pin names/aliases.
        match_regex = criteria.pop("match_regex", False) or self.match_pin_regex

        # If no pin identifiers were given, then use a wildcard that will
        # select all pins.
        if not io_ids:
            io_ids = [".*"]
            match_regex = True

        # An interface doesn't have pins, so set pin slice bounds to zero.
        min_pin, max_pin = 0, 0

        # Get I/O entries.
        io_types = (Net, Pin, NetPinList, Bus)
        ios = [io for io in self.values() if isinstance(io, io_types)]

        # Use this for looking up the dict key using the id of a given I/O.
        id_to_key = {id(v): k for k, v in self.items()}

        # Go through the I/O entries and find the ones selected by the IDs.
        selected_ios = NetPinList()
        for io_id in expand_indices(min_pin, max_pin, match_regex, *io_ids):

            # Look for an exact match of the I/O key name with the current ID.
            try:
                io = dict.__getitem__(self, io_id)
            except KeyError:
                # No exact match on I/O key name, so keep looking below.
                pass
            else:
                # Add exact match to the list of selected I/Os and go to the next ID.
                selected_ios.append(io)
                continue

            # Check I/O aliases for an exact match with the current ID.
            tmp_ios = filter_list(ios, aliases=io_id, do_str_match=True, **criteria)
            for io in tmp_ios:
                selected_ios.append(io)
            if tmp_ios:
                # Found exact match between alias and ID, so done with this ID and go to next ID.
                continue

            # Skip regex matching if not enabled.
            if not match_regex:
                continue

            # OK, ID doesn't exactly match an I/O name or alias. Does it match as a regex?
            tmp_ios = filter_list(ios, aliases=Alias(io_id), **criteria)
            for io in tmp_ios:
                selected_ios.append(io)

        # Return list of I/Os that were selected by the IDs.
        return list_or_scalar(selected_ios)

    def __setitem__(self, key, value):
        """
        Set a dictionary entry and attribute with the same name.
        
        This method handles assignments to dictionary keys, ensuring that
        attribute access and dictionary access remain synchronized.
        
        Args:
            key (str): The key to assign to.
            value: The value to assign.
        """
        if from_iadd(value):
            # The += flag in the values are no longer needed.
            rmv_iadd(value)
        else:
            # This is for a straight assignment of value to key.
            setattr(self, key, value)

    def __iadd__(self, other_intfc):
        """
        Connect this interface to another interface using the += operator.
        
        This connects matching nets and buses between the two interfaces.
        
        Args:
            other_intfc (Interface): The interface to connect to this one.
            
        Returns:
            Interface: This interface with connections to the other interface.
        """
        return self.connect(other_intfc)

    def connect(self, other_intfc):
        """
        Connect this interface to another interface.
        
        This method connects nets and buses in this interface to those
        with matching names in another interface.
        
        Args:
            other_intfc (Interface): The interface to connect to this one.
            
        Returns:
            Interface: This interface with connections to the other interface.
            
        Examples:
            >>> intf1 = Interface(a=Net('a'), b=Net('b'))
            >>> intf2 = Interface(a=Net('a2'), b=Net('b2'))
            >>> intf1.connect(intf2)  # Connect matching nets
            >>> # Alternative: intf1 += intf2
        """

        # Connect the nets/buses of this interface to the nets/buses of the other interface.
        for k,v in self.unexpio.items():
            if isinstance(v, (Net, Bus, Pin)):
                if k in other_intfc:
                    self[k] += other_intfc[k]

        # Set the flag to indicate this result came from the += operator.
        set_iadd(self, True)

        return self



from collections import defaultdict
from skidl import Pin, Part, Alias, SchLib, SKIDL, TEMPLATE

from skidl.pin import pin_types

SKIDL_lib_version = '0.0.1'

Interface_HDMI = SchLib(tool=SKIDL).add_parts(*[
        Part(**{ 'name':'ADV7611', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'ADV7611'}), 'ref_prefix':'U', 'fplist':['Package_QFP:LQFP-64-1EP_10x10mm_P0.5mm_EP5x5mm'], 'footprint':'Package_QFP:LQFP-64-1EP_10x10mm_P0.5mm_EP5x5mm', 'keywords':'hdmi', 'description':'', 'datasheet':'https://www.analog.com/media/en/technical-documentation/data-sheets/adv7611.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_HDMI.kicad_sym\nADV7611\n\nhdmi', 'pins':[
            Pin(num='1',name='HPA_A/INT2',func=pin_types.BIDIR,unit=1),
            Pin(num='10',name='RXA_1+',func=pin_types.INPUT,unit=1),
            Pin(num='11',name='TVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='12',name='RXA_2-',func=pin_types.INPUT,unit=1),
            Pin(num='13',name='RXA_2+',func=pin_types.INPUT,unit=1),
            Pin(num='14',name='CVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='15',name='P23',func=pin_types.OUTPUT,unit=1),
            Pin(num='16',name='P22',func=pin_types.OUTPUT,unit=1),
            Pin(num='17',name='P21',func=pin_types.OUTPUT,unit=1),
            Pin(num='18',name='P20',func=pin_types.OUTPUT,unit=1),
            Pin(num='19',name='P19',func=pin_types.OUTPUT,unit=1),
            Pin(num='2',name='CVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='P18',func=pin_types.OUTPUT,unit=1),
            Pin(num='21',name='P17',func=pin_types.OUTPUT,unit=1),
            Pin(num='22',name='P16',func=pin_types.OUTPUT,unit=1),
            Pin(num='23',name='DVDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='24',name='DVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='25',name='LLC',func=pin_types.OUTPUT,unit=1),
            Pin(num='26',name='P15',func=pin_types.OUTPUT,unit=1),
            Pin(num='27',name='P14',func=pin_types.OUTPUT,unit=1),
            Pin(num='28',name='P13',func=pin_types.OUTPUT,unit=1),
            Pin(num='29',name='P12',func=pin_types.OUTPUT,unit=1),
            Pin(num='3',name='RXA_C-',func=pin_types.INPUT,unit=1),
            Pin(num='30',name='P11',func=pin_types.OUTPUT,unit=1),
            Pin(num='31',name='P10',func=pin_types.OUTPUT,unit=1),
            Pin(num='32',name='P9',func=pin_types.OUTPUT,unit=1),
            Pin(num='33',name='P8',func=pin_types.OUTPUT,unit=1),
            Pin(num='34',name='DVDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='35',name='P7',func=pin_types.OUTPUT,unit=1),
            Pin(num='36',name='P6',func=pin_types.OUTPUT,unit=1),
            Pin(num='37',name='P5',func=pin_types.OUTPUT,unit=1),
            Pin(num='38',name='P4',func=pin_types.OUTPUT,unit=1),
            Pin(num='39',name='P3',func=pin_types.OUTPUT,unit=1),
            Pin(num='4',name='RXA_C+',func=pin_types.INPUT,unit=1),
            Pin(num='40',name='DVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='41',name='P2',func=pin_types.OUTPUT,unit=1),
            Pin(num='42',name='P1',func=pin_types.OUTPUT,unit=1),
            Pin(num='43',name='P0',func=pin_types.OUTPUT,unit=1),
            Pin(num='44',name='DVDDIO',func=pin_types.PWRIN,unit=1),
            Pin(num='45',name='DE',func=pin_types.OUTPUT,unit=1),
            Pin(num='46',name='HS',func=pin_types.OUTPUT,unit=1),
            Pin(num='47',name='VS/FIELD/ALSB',func=pin_types.BIDIR,unit=1),
            Pin(num='48',name='AP',func=pin_types.OUTPUT,unit=1),
            Pin(num='49',name='SCLK/INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='5',name='TVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='50',name='LRCLK',func=pin_types.OUTPUT,unit=1),
            Pin(num='51',name='MCLK/INT2',func=pin_types.OUTPUT,unit=1),
            Pin(num='52',name='DVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='53',name='SCL',func=pin_types.INPUT,unit=1),
            Pin(num='54',name='SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='55',name='INT1',func=pin_types.OUTPUT,unit=1),
            Pin(num='56',name='~{RESET}',func=pin_types.INPUT,unit=1),
            Pin(num='57',name='PVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='58',name='XTALP',func=pin_types.INPUT,unit=1),
            Pin(num='59',name='XTALN',func=pin_types.INPUT,unit=1),
            Pin(num='6',name='RXA_0-',func=pin_types.INPUT,unit=1),
            Pin(num='60',name='DVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='61',name='CEC',func=pin_types.BIDIR,unit=1),
            Pin(num='62',name='DDCA_SCL',func=pin_types.INPUT,unit=1),
            Pin(num='63',name='DDCA_SDA',func=pin_types.BIDIR,unit=1),
            Pin(num='64',name='RXA_5V',func=pin_types.INPUT,unit=1),
            Pin(num='65',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='7',name='RXA_0+',func=pin_types.INPUT,unit=1),
            Pin(num='8',name='TVDD',func=pin_types.PWRIN,unit=1),
            Pin(num='9',name='RXA_1-',func=pin_types.INPUT,unit=1)], 'unit_defs':[] }),
        Part(**{ 'name':'TPD12S520DBT', 'dest':TEMPLATE, 'tool':SKIDL, 'aliases':Alias({'TPD12S520DBT'}), 'ref_prefix':'U', 'fplist':['Package_SO:TSSOP-38_4.4x9.7mm_P0.5mm'], 'footprint':'Package_SO:TSSOP-38_4.4x9.7mm_P0.5mm', 'keywords':'hdmi', 'description':'', 'datasheet':'http://www.ti.com/lit/ds/symlink/tpd12s520.pdf', 'search_text':'/usr/share/kicad/symbols/Interface_HDMI.kicad_sym\nTPD12S520DBT\n\nhdmi', 'pins':[
            Pin(num='1',name='5V_SUPPLY',func=pin_types.PWRIN,unit=1),
            Pin(num='10',name='TMDS_D0+',func=pin_types.PASSIVE,unit=1),
            Pin(num='11',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='12',name='TMDS_D0-',func=pin_types.PASSIVE,unit=1),
            Pin(num='13',name='TMDS_CK+',func=pin_types.PASSIVE,unit=1),
            Pin(num='14',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='15',name='TMDS_CK-',func=pin_types.PASSIVE,unit=1),
            Pin(num='16',name='CE_REMOTE_IN',func=pin_types.BIDIR,unit=1),
            Pin(num='17',name='DDC_CLK_IN',func=pin_types.BIDIR,unit=1),
            Pin(num='18',name='DDC_DAT_IN',func=pin_types.BIDIR,unit=1),
            Pin(num='19',name='HOTPLUG_DET_IN',func=pin_types.BIDIR,unit=1),
            Pin(num='2',name='LV_SUPPLY',func=pin_types.PWRIN,unit=1),
            Pin(num='20',name='HOTPLUG_DET_OUT',func=pin_types.BIDIR,unit=1),
            Pin(num='21',name='DDC_DAT_OUT',func=pin_types.BIDIR,unit=1),
            Pin(num='22',name='DDC_CLK_OUT',func=pin_types.BIDIR,unit=1),
            Pin(num='23',name='CE_REMOTE_OUT',func=pin_types.BIDIR,unit=1),
            Pin(num='24',name='TMDS_CK-',func=pin_types.PASSIVE,unit=1),
            Pin(num='25',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='26',name='TMDS_CK+',func=pin_types.PASSIVE,unit=1),
            Pin(num='27',name='TMDS_D0-',func=pin_types.PASSIVE,unit=1),
            Pin(num='28',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='29',name='TMDS_D0+',func=pin_types.PASSIVE,unit=1),
            Pin(num='3',name='GND',func=pin_types.PWRIN,unit=1),
            Pin(num='30',name='TMDS_D1-',func=pin_types.PASSIVE,unit=1),
            Pin(num='31',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='32',name='TMDS_D1+',func=pin_types.PASSIVE,unit=1),
            Pin(num='33',name='TMDS_D2-',func=pin_types.PASSIVE,unit=1),
            Pin(num='34',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='35',name='TMDS_D2+',func=pin_types.PASSIVE,unit=1),
            Pin(num='36',name='GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='37',name='ESD_BYP',func=pin_types.PASSIVE,unit=1),
            Pin(num='38',name='NC',func=pin_types.NOCONNECT,unit=1),
            Pin(num='4',name='TMDS_D2+',func=pin_types.PASSIVE,unit=1),
            Pin(num='5',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='6',name='TMDS_D2-',func=pin_types.PASSIVE,unit=1),
            Pin(num='7',name='TMDS_D1+',func=pin_types.PASSIVE,unit=1),
            Pin(num='8',name='TMDS_GND',func=pin_types.PASSIVE,unit=1),
            Pin(num='9',name='TMDS_D1-',func=pin_types.PASSIVE,unit=1)], 'unit_defs':[] })])